/*
 * ADC-LF.c
 */ 

#define F_CPU 3333333
#define BAUD_9600 4*F_CPU/9600
#define TX_PORT	PORTB
#define TX_PIN 0
char uart_str[64];

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

/*
In this exercise we will set up and use the ADC (analog-digital-converter). 
You only need to implement adc_init, adc_read and the rest of main.
The rest of the code you see is used for UART transmission, so the result after reading analog input can be transmitted.

If you want to do something else than just transmit the result, feel free to do so!
For example, PWM can be used to dim LEDs according to the input voltage.
*/ 



void uart_init(unsigned long baud){
	//Sets up registers for UART
	
	TX_PORT.OUTSET = (1 << TX_PIN); 	//Setting up TX pin as output
	TX_PORT.DIRSET = (1 << TX_PIN); 	//Setting up TX pin as output
	
	//Set baud rate register
	USART3.BAUDL = (uint8_t) baud;			//Set baud rate without shifting to get the 8 low bits
	USART3.BAUDH = (uint8_t)(baud >> 8);	//Shift register right by 8 bits to get the 8 high bits
	
	
	//USART.CTRLC CMODE bits default to async, 1 stop bit, 8 bit character size
	//Since all bits are default 0 we only need to change the character size part
	USART3.CTRLC = (0x3 << USART_CHSIZE0_bp);
	
	//Enable RX and TX
	USART3.CTRLB = (1 << USART_RXEN_bp) | (1 << USART_TXEN_bp);

	//Enable interrupts on incoming data
	USART3.CTRLA |= (1 << USART_RXCIE_bp); 
}

void uart_transmit(char data){	
	//Sends a character over UART
	
	//First we should check that there isn't already data being sent
	//	if there is, we should probably wait for it to finish first
	while (!(USART3.STATUS & (1 << USART_DREIF_bp))){
		//wait for previous transmit to finish
	};
	
	//Put our new data into tx data register
	USART3.TXDATAL = data;
}


//To send a string we can do this
void uart_transmit_string(char* data) {
	while (*data != '\0') {
		uart_transmit(*data);
		data++;
	}
}

//helper function for converting data from adc into a nice readable format, and sending it over UART
void uart_print_voltage(uint16_t data) {
    uint16_t data2 = data / 4; 
    uint8_t voltage_integer = data2*323ul/100000ul; //This is a lot faster than floats. Compiler magic
	uint8_t voltage_fraction = (data2*323ul%100000ul)/1000;

	sprintf(uart_str, "Voltage: %u.%02u V\n",voltage_integer,voltage_fraction); //format string and put voltage  
    uart_transmit_string(uart_str);
}

void adc_init() {
	/*
        This function sets up the required registers to initialize the ADC
		Not as much help this time around.
		We want our ADC to run in 10 bit mode using the supply voltage (VDD) as reference.
		The rest is up to you to find out.
	*/
    /* YOUR CODE */


}

uint16_t adc_read(uint8_t channel) { 
    //This function takes in a channel and returns the result of the ADC operation on that channel
	//Keep in mind an ADC operation takes a little time
    /* YOUR CODE */


}

int main(void)
{
	//Initialize the UART with our function. 
	//We will be using a baudrate of 9600 (defined as BAUD_9600 at the top of the file)
	uart_init(BAUD_9600);
    adc_init();
	sei(); //Enable inerrupt, important for anything here to work
	
    //read analog input using polling, and display the result using uart_print_voltage (or do something else with it?)
    /* YOUR CODE */


}


//Interrupt service routine for the UART receiver. Doesn't actually do anything with the received data.
ISR (USART3_RXC_vect) {
	//In the interrupt we will read the data in the receive buffer
	
	//First we should check that new data has arrived the receive buffer
	while (!(USART3.STATUS & (1 << USART_RXCIF_bp))){
		//wait for previous transmit to finish
	};
	
	//Store the data in a temporarily variable
	uint8_t tmp = USART3.RXDATAL; 
}